<?php
/**
*
* @package install
* @version $Id: convert_ipb13.php,v 0.2 2008/02/16 10:00:00 maat Exp $
* @copyright (c) 2008 grenouille.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License V3
*
*/

/**
* NOTE to potential convertor authors. Please use this file to get
* familiar with the structure since we added some bare explanations here.
*
* Since this file gets included more than once on one page you are not able to add functions to it.
* Instead use a functions_ file.
*
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

include($phpbb_root_path . 'config.' . $phpEx);
unset($dbpasswd);

/**
* $convertor_data provides some basic information about this convertor which is
* used on the initial list of convertors and to populate the default settings
*/
$convertor_data = array(
	'forum_name'	=> 'IPB 1.3',
	'version'		=> '0.4.dev',
	'phpbb_version'	=> '3.0.X',
	'author'		=> '<a href="http://www.grenouille.com/">Maât <maat@grenouille.com></a>',
	'dbms'			=> $dbms,
	'dbhost'		=> $dbhost,
	'dbport'		=> $dbport,
	'dbuser'		=> $dbuser,
	'dbpasswd'		=> '',
	'dbname'		=> $dbname,
	'charset'		=> 'cp1252',
	'table_prefix'	=> 'ibf_',
	'forum_path'	=> '../ipb13',
	'author_notes'	=> 'First trial',
);

/**
* $tables is a list of the tables (minus prefix) which we expect to find in the
* source forum. It is used to guess the prefix if the specified prefix is incorrect
*/
$tables = array(
	'admin_logs',
	'admin_sessions',
	'badwords',
	'cache_store',
	'calendar_events',
	'categories',
	'contacts',
	'css',
	'email_logs',
	'emoticons',
	'faq',
	'forum_perms',
	'forums',
	'forum_tracker',
	'groups',
	'languages',
	'macro',
	'macro_name',
	'member_extra',
	'members',
	'messages',
	'moderator_logs',
	'moderators',
	'pfields_content',
	'pfields_data',
	'polls',
	'posts',
	'reg_antispam',
	'search_results',
	'sessions',
	'skins',
	'skin_templates',
	'spider_logs',
	'stats',
	'subscription_currency',
	'subscription_extra',
	'subscription_logs',
	'subscription_methods',
	'subscriptions',
	'subscription_trans',
	'templates',
	'titles',
	'tmpl_names',
	'topic_mmod',
	'topics',
	'tracker',
	'validating',
	'voters',
	'warn_logs',
);

/**
* $config_schema details how the board configuration information is stored in the source forum.
*
* 'table_format' can take the value 'file' to indicate a config file. In this case array_name
* is set to indicate the name of the array the config values are stored in
* 'table_format' can be an array if the values are stored in a table which is an assosciative array
* (as per phpBB 2.0.x)
* If left empty, values are assumed to be stored in a table where each config setting is
* a column (as per phpBB 1.x)
*
* In either of the latter cases 'table_name' indicates the name of the table in the database
*
* 'settings' is an array which maps the name of the config directive in the source forum
* to the config directive in phpBB3. It can either be a direct mapping or use a function.
* Please note that the contents of the old config value are passed to the function, therefore
* an in-built function requiring the variable passed by reference is not able to be used. Since
* empty() is such a function we created the function is_empty() to be used instead.
*/
$config_schema = array(
	'table_format'          => 'file',
        'filename'              => 'conf_global.php',
	'array_name'            => 'INFO',                              //   TODO : check if 'INFO' value is correct (perhaps 'config' like with phpBB2 ?).
	'settings'              => array(
		'msg_allow_code'	=>  'allow_bbcode',		//   OK ?
		'auth_allow_dnames'	=>  'allow_namechange',		//   OK
		'avatars_on'		=>  'allow_avatar_local',	//   OK
		'avatar_url'		=>  'allow_avatar_remote',	//   OK
		'avatar_ext'		=>  'allow_avatar_upload',	//   NEEDS CHECK
		'board_offline'		=>  'board_disable',		//   OK
		'board_name'		=>  'sitename',			//   OK
		'board_desc'		=>  'site_desc',		//   OK
		'session_expiration'	=>  'session_length',		//   OK
		'display_max_posts'	=>  'posts_per_page',		//   OK ?
		'display_max_topics'	=>  'topics_per_page',		//   OK ?
		'email_out'		=>  'board_email_form',		//   OK ?
		'allow_skins'		=>  'override_user_style',	//   OK
		'hot_topic'		=>  'hot_threshold',		//   OK
		'max_poll_choices'	=>  'max_poll_options',		//   OK
		'max_sig_length'	=>  'max_sig_chars',		//   OK
		'smtp_port'		=>  'smtp_delivery',		//   OK
		'smtp_host'		=>  'smtp_host',		//   OK
		'smtp_user'		=>  'smtp_username',		//   OK
		'smtp_pass'		=>  'smtp_password',		//   OK
		'flood_control'		=>  'flood_interval',		//   OK
		'avup_size_max'		=>  'avatar_filesize',		//   OK
		'avatar_max_width'	=>  'avatar_max_width',		//   Needs a function to split avatar_dims or avatar_def
		'avatar_max_height'	=>  'avatar_max_height',	//   Needs a function to split avatar_dims or avatar_def
		'csite_article_date'	=>  'default_dateformat',	//   OK
		'time_offset'		=>  'board_timezone',		//   OK
		'disable_gzip'		=>  'not(gzip_compress)',	//   OK
		'board_start'		=>  'board_startdate',		//   OK perhaps needs conversion
	)
);

/**
* $test_file is the name of a file which is present on the source
* forum which can be used to check that the path specified by the
* user was correct
*/
$test_file = 'conf_global.php';

/**
* If this is set then we are not generating the first page of information but getting the conversion information.
*/
if (!$get_info)
{

	$src_db->sql_return_on_error(false);

	// Overwrite maximum avatar width/height
	@define('DEFAULT_AVATAR_X_CUSTOM', get_config_value('avatar_max_width'));
	@define('DEFAULT_AVATAR_Y_CUSTOM', get_config_value('avatar_max_height'));

/**
*	Description on how to use the convertor framework.
*
*	'schema' Syntax Description
*		-> 'target'			=> Target Table. If not specified the next table will be handled
*		-> 'primary'		=> Primary Key. If this is specified then this table is processed in batches
*		-> 'query_first'	=> array('target' or 'src', Query to execute before beginning the process
*								(if more than one then specified as array))
*		-> 'function_first'	=> Function to execute before beginning the process (if more than one then specified as array)
*								(This is mostly useful if variables need to be given to the converting process)
*		-> 'test_file'		=> This is not used at the moment but should be filled with a file from the old installation
*
*		// DB Functions
*		'distinct'	=> Add DISTINCT to the select query
*		'where'		=> Add WHERE to the select query
*		'group_by'	=> Add GROUP BY to the select query
*		'left_join'	=> Add LEFT JOIN to the select query (if more than one joins specified as array)
*		'having'	=> Add HAVING to the select query
*
*		// DB INSERT array
*		This one consist of three parameters
*		First Parameter:
*							The key need to be filled within the target table
*							If this is empty, the target table gets not assigned the source value
*		Second Parameter:
*							Source value. If the first parameter is specified, it will be assigned this value.
*							If the first parameter is empty, this only gets added to the select query
*		Third Parameter:
*							Custom Function. Function to execute while storing source value into target table.
*							The functions return value get stored.
*							The function parameter consist of the value of the second parameter.
*
*							types:
*								- empty string == execute nothing
*								- string == function to execute
*								- array == complex execution instructions
*		
*		Complex execution instructions:
*		@todo test complex execution instructions - in theory they will work fine
*
*							By defining an array as the third parameter you are able to define some statements to be executed. The key
*							is defining what to execute, numbers can be appended...
*
*							'function' => execute function
*							'execute' => run code, whereby all occurrences of {VALUE} get replaced by the last returned value.
*										The result *must* be assigned/stored to {RESULT}.
*							'typecast'	=> typecast value
*
*							The returned variables will be made always available to the next function to continue to work with.
*
*							example (variable inputted is an integer of 1):
*
*							array(
*								'function1'		=> 'increment_by_one',		// returned variable is 2
*								'typecast'		=> 'string',				// typecast variable to be a string
*								'execute'		=> '{RESULT} = {VALUE} . ' is good';', // returned variable is '2 is good'
*								'function2'		=> 'replace_good_with_bad',				// returned variable is '2 is bad'
*							),
*
*/

	$convertor = array(
		'test_file'            => $test_file,
		'avatar_path'          => 'uploads/',
		'upload_path'          =>  'uploads/',
		
		
		// We empty some tables to have clean data available
		'query_first'         => array(
			array('target', $convert->truncate_statement . SEARCH_RESULTS_TABLE),
			array('target', $convert->truncate_statement . SEARCH_WORDLIST_TABLE),
			array('target', $convert->truncate_statement . SEARCH_WORDMATCH_TABLE),
			array('target', $convert->truncate_statement . LOG_TABLE),
		),
		
		'execute_first'   => '
			phpbb_insert_forums();
		',
		
		'execute_last'   => array('
			update_folder_pm_count();
		', '
			update_unread_count();
		', '
			phpbb_convert_authentication();
		'),
		
		'schema' => array(
		
			array(
				'target'         => RANKS_TABLE,
				'query_first'    => array('target', $convert->truncate_statement . RANKS_TABLE),
				'autoincrement'  => 'rank_id',
			
				array('rank_id',             'titles.id',                      ''),
				array('rank_title',          'titles.title',                   array('function1' => 'phpbb_set_default_encoding', 'function2' => 'utf8_htmlspecialchars')),
				array('rank_min',            'titles.posts',                   array('typecast' => 'int', 'execute' => '{RESULT} = ({VALUE}[0] < 0) ? 0 : {VALUE}[0];')),
				array('rank_special',        0,                                ''),
				array('rank_image',          '',                               'import_rank'),
			),
			array(
				'target'         => RANKS_TABLE,
			  'distinct'       => True,
				'autoincrement'  => 'rank_id',
			
				array('rank_id',             'null',                      ''),
				array('rank_title',          'members.title',                   array('function1' => 'phpbb_set_default_encoding')),
				array('rank_min',            0,                   ''),
				array('rank_special',        1,                                ''),
				array('rank_image',          '',                               ''),

					'where'         => 'members.title != ""',
				
				),

			array(
				'target'         => SMILIES_TABLE,
				'query_first'    => array('target', $convert->truncate_statement . SMILIES_TABLE),
				'autoincrement'  => 'smiley_id',
			
				array('smiley_id',            'null',                                           ''),
				array('code',                  'emoticons.typed',                         ''),
				array('emotion',              'emoticons.typed',                        ''),
				array('smiley_url',            'emoticons.image',                      'copy_smiley_from_ipb_dir'),
				array('smiley_width',         'emoticons.image',                     'get_ipb_smiley_width'),
				array('smiley_height',        'emoticons.image',                     'get_ipb_smiley_height'),
				array('smiley_order',          0,                                            ''),
				array('display_on_posting',          'emoticons.clickable',        ''),
				),
				
			array(
				'target'          => GROUPS_TABLE,
				'autoincrement'   => 'group_id',
				'query_first'     => array('target', $convert->truncate_statement . GROUPS_TABLE),
			
				array('group_id',            'groups.g_id',                    ''),
				array('group_type',          0,                                'phpbb_convert_group_type'),
				array('group_display',       0,                                ''),
				array('group_legend',        0,                                ''),
				array('group_name',          'groups.g_title',                 'phpbb_convert_group_name'), // phpbb_set_encoding called in phpbb_convert_group_name
				array('group_desc',          '',                               'phpbb_set_encoding'),

				'where'         => 'groups.g_id > 5',                                                       // groups 1 to 5 are ipb administrative standard groups
			),
		
			array(
				'target'          => USER_GROUP_TABLE,
				'query_first'     => array('target', $convert->truncate_statement . USER_GROUP_TABLE),
				'execute_first'   => '
				add_default_groups();
				',
			
				array('group_id',            'groups.g_id',                    ''),
				array('user_id',             'members.id',                     'phpbb_user_id'),
				array('group_leader',        1,                                ''),
				array('user_pending',        0,                                ''),

				'where'         => 'members.mgroup = groups.g_id AND groups.g_id > 5',
			),

			array(
				'target'           => USERS_TABLE,
				'primary'          => 'members.id',
				'autoincrement'    => 'user_id',
				'query_first'      => array(
					array('target', 'DELETE FROM ' . USERS_TABLE . ' WHERE user_id <> ' . ANONYMOUS),
					array('target', $convert->truncate_statement . BOTS_TABLE)
				),
			
				'execute_last'   => '
				remove_invalid_users();
				',
			
				array('user_id',                   'members.id',               'phpbb_user_id'),
				array('',                          'members.id AS poster_id',  'phpbb_user_id'),

				array('user_type',                 'members.mgroup',           'ipb13_user_is_active'),

				array('group_id',                  'members.mgroup',           'phpbb_set_primary_group'),
				array('user_regdate',              'members.joined',           ''),
				array('username',                  'members.name',             'phpbb_set_default_encoding'), // recode to utf8 with default lang
				array('username_clean',            'members.name',             'phpbb_set_clean_name'),
				array('user_password',             'members.password',                         ''),
				array('user_pass_convert',         1,                          ''),
				array('user_posts',                'members.posts',            ''),
				array('user_email',                'members.email',            ''),
				array('user_email_hash',           'members.email',            'gen_email_hash'),
				array('user_birthday',             'members.id',               'phpbb_birthday'),
				array('user_lastvisit',            'members.last_activity',    ''),
				array('user_lastmark',             'members.last_activity',    ''),
				array('user_lang',                 $config['default_lang'],    ''),

				array('user_timezone',             'members.time_offset',      ''),
				array('user_dateformat',           '',                         array('function1' => 'phpbb_set_encoding', 'function2' => 'fill_dateformat')),
				array('user_inactive_reason',      '',                         'phpbb_inactive_reason'),
				array('user_inactive_time',        '',                         'phpbb_inactive_time'),
			
				array('user_interests',            '',                         ''),
				array('user_occ',                  '',                         array('function1' => 'phpbb_set_encoding')),
				array('user_website',              'members.website',          'validate_website'),
				array('user_jabber',               '',                         ''),
				array('user_msnm',                 'members.msnname',          array('function1' => 'phpbb_set_encoding')),
				array('user_yim',                  'members.yahoo',            array('function1' => 'phpbb_set_encoding')),
				array('user_aim',                  'members.aim_name',         array('function1' => 'phpbb_set_encoding')),
				array('user_icq',                  'members.icq_number',       array('function1' => 'phpbb_set_encoding')),
				array('user_from',                 'members.location',         array('function1' => 'phpbb_set_encoding')),

				array('user_rank',                 'members.title',            'phpbb_get_user_rank'),
				array('user_permissions',          '',                         ''),
			
				array('user_avatar',               'members.avatar',                'phpbb_import_avatar'),
				array('user_avatar_type',          'members.avatar',             'ipb_avatar_type'),
				array('user_avatar_width',         'members.avatar_size',      'phpbb_get_avatar_width'),
				array('user_avatar_height',        'members.avatar_size',      'phpbb_get_avatar_height'),
			
				array('user_new_privmsg',          '',                         ''),
				array('user_unread_privmsg',       0,                          ''), //'users.user_unread_privmsg'
				array('user_last_privmsg',         '',                         ''),
				array('user_emailtime',            '',                         'null_to_zero'),
				array('user_notify',               '',                         ''),
				array('user_notify_pm',            '',                         ''),
				array('user_notify_type',          NOTIFY_EMAIL,               ''),
				array('user_allow_pm',             '',                         ''),
				array('user_allow_viewonline',     '',                         ''),
				array('user_allow_viewemail',      '',                         ''),
				array('user_actkey',               '',                         ''),
				array('user_newpasswd',            '',                         ''), // Users need to re-request their password...
				array('user_style',                $config['default_style'],   ''),
			
				array('user_options',              '',                         'set_user_options'),
			
				array('user_sig_bbcode_uid',       'members.joined',                         'make_uid'),
				array('user_sig',                  'members.signature',        'phpbb_prepare_message'),

				array('user_sig_bbcode_bitfield',  '',                         'get_bbcode_bitfield'),
			
				'where'         => 'members.id <> -1 ',
				//'left_join'     => 'member_extra on member_extra.id=members.id ',
			),

			array(
				'target'         => TOPICS_TABLE,
				'query_first'    => array('target', $convert->truncate_statement . TOPICS_TABLE),
				'primary'        => 'topics.tid',
				'autoincrement'  => 'topic_id',

				'execute_last'   => '
				update_ipb_polls();
				',

				array('topic_id',                'topics.tid',                        ''),
				array('forum_id',              'topics.forum_id',                ''),
				array('icon_id',                 0,                                     ''),
				array('topic_poster',         'topics.starter_id AS poster_id', 'phpbb_user_id'),
				array('topic_title',             'topics.title',                       'phpbb_set_encoding'),
				array('topic_time',            'topics.start_date',              ''),
				array('topic_views',          'topics.views',                     ''),
				array('topic_replies',        'topics.posts',                     ''),
				array('topic_replies_real', 'topics.posts',                     ''),
				array('topic_last_post_id', 'topics.last_poster_id',          ''),
				array('topic_status',         'topics.state',                       'phpbb_convert_status'),
				array('topic_moved_id',     'topics.moved_to',                'phpbb_convert_moved_id'),
				array('topic_type',             'topics.pinned',                    'phpbb_convert_topic_type'),
				array('topic_first_post_id',  '',                                       ''),
			),

			array(
			'target'      => PRIVMSGS_TABLE,
			'primary'      => 'messages.msg_id',
			'autoincrement'   => 'msg_id',
			'query_first'   => array(
			array('target', $convert->truncate_statement . PRIVMSGS_TABLE),
			array('target', $convert->truncate_statement . PRIVMSGS_RULES_TABLE),
			),
		
			'execute_first'   => '
			$config["max_post_chars"] = -1;
			$config["max_quote_depth"] = 0;
			',
		
			array('msg_id',               'messages.msg_id',            ''),
			array('root_level',            0,                           ''),
			array('author_id',            'messages.from_id',   'phpbb_user_id'),
			array('icon_id',            0,                           ''),
			//array('author_ip',            '',            ''),
			array('message_time',         'messages.msg_date',         ''),
			array('enable_bbcode',         1,   ''),

			array('enable_smilies',         1,   ''),
			array('enable_magic_url',      1,                           ''),
			array('enable_sig',         '',      ''),
			array('message_subject',      'messages.title',      'phpbb_set_encoding'), // Already specialchared in 2.0.x
			array('message_attachment',      '', ''),
			array('message_edit_reason',   '',                           ''),
			array('message_edit_user',      0,                           ''),
			array('message_edit_time',      0,                           ''),
			array('message_edit_count',      0,                           ''),
		
			array('bbcode_uid',         'messages.msg_date',   'make_uid'),
			array('message_text',         'messages.message',         'phpbb_prepare_message'),
			array('',            '',         ''),
			array('bbcode_bitfield',      '',                              'get_bbcode_bitfield'),
			array('to_address',         'messages.recipient_id',         'phpbb_privmsgs_to_userid'),
			array('bcc_address',         '',                              ''),

			),
		
// 			array(
// 			'target'      => PRIVMSGS_FOLDER_TABLE,
// 			'primary'      => 'members.id',
// 			'query_first'   => array('target', $convert->truncate_statement . PRIVMSGS_FOLDER_TABLE),
// 		
// 			array('user_id', 'members.id',                  'phpbb_user_id'),
// 			array('folder_name',   $user->lang['CONV_SAVED_MESSAGES'],      ''),
// 			array('pm_count',            0,                              ''),
// 		
// 			'where'         => 'members.id <> -1',
// 			),

			// Inbox
			array(
			'target'      => PRIVMSGS_TO_TABLE,
			'primary'      => 'messages.msg_id',
			'query_first'   => array('target', $convert->truncate_statement . PRIVMSGS_TO_TABLE),
		
			array('msg_id',               'messages.msg_id',               ''),
			array('user_id',            'messages.recipient_id',         'phpbb_user_id'),
			array('author_id',            'messages.from_id',      'phpbb_user_id'),
			array('pm_deleted',            0,                              ''),
			array('pm_new',               '',            'phpbb_new_pm'),
			array('pm_unread',            '',            'phpbb_unread_pm'),
			array('pm_replied',            0,                              ''),
			array('pm_marked',            0,                              ''),
			array('pm_forwarded',         0,                              ''),
			array('folder_id',            PRIVMSGS_INBOX,                     ''),
		
			'where'         => 'messages.vid = \'in\'',
			),
				
			// Sentbox
			array(
			'target'      => PRIVMSGS_TO_TABLE,
			'primary'      => 'messages.msg_id',
		
			array('msg_id',               'messages.msg_id',               ''),
			array('user_id',            'messages.from_id',         'phpbb_user_id'),
			array('author_id',            'messages.from_id',      'phpbb_user_id'),
			array('pm_deleted',            0,                              ''),
			array('pm_new',               '',            'phpbb_new_pm'),
			array('pm_unread',            '',            'phpbb_unread_pm'),
			array('pm_replied',            0,                              ''),
			array('pm_marked',            0,                              ''),
			array('pm_forwarded',         0,                              ''),
			array('folder_id',            PRIVMSGS_SENTBOX,                     ''),
		
			'where'         => 'messages.vid = \'sent\'',
			),

			array(
 				'target'        => ATTACHMENTS_TABLE,
 				'primary'       => 'posts.pid',
 				'autoincrement' => 'attach_id',
 				'query_first'   => array('target', $convert->truncate_statement . ATTACHMENTS_TABLE),
 				'execute_first' => '',
 				'where' =>  'posts.attach_id != \'\'',

				array('attach_id',             'null',                  ''), # Autoincement
 				array('post_msg_id',           'posts.pid',             ''),
 				array('topic_id',              'posts.topic_id',        ''),
 				array('in_message',            0,                       ''),
 				array('poster_id',             'posts.author_id',       'phpbb_user_id'),
 				array('is_orphan',             0,                       ''),
 				array('physical_filename',     'posts.attach_id',       'phpbb_import_attachement'),
 				array('real_filename',         'posts.attach_file',     ''),
 
 				array('download_count',        'posts.attach_hits',     ''),
 				array('attach_comment',        '',         ''),
 				array('extension',             'posts.attach_file',     'phpbb_get_attachement_extension'),
 				array('mimetype',              'posts.attach_type',     'phpbb_get_attachement_mimetype'),
 				array('filesize',              'posts.attach_id',       'phpbb_get_attachement_filesize'),
 				array('filetime',              'posts.post_date',       ''),
 				array('thumbnail',             '',                      ''),
 
 			),
 			
 			array(
 				'target'        => POSTS_TABLE,
 				'primary'       => 'posts.pid',
 				'autoincrement' => 'post_id',
 				'query_first'   => array('target', $convert->truncate_statement . POSTS_TABLE),
 				'execute_first' => '
 				$config["max_post_chars"] = -1;
 				$config["max_quote_depth"] = 0;
 				',
 			
 				array('post_id',             'posts.pid',             ''),
 				array('topic_id',            'posts.topic_id',        ''),
 				array('forum_id',            '',                      ''),
 				array('poster_id',           'posts.author_id',       'phpbb_user_id'),
 				array('icon_id',             0,                       ''),
 				array('poster_ip',           'posts.ip_address',      'decode_ip'),
 				array('post_time',           'posts.post_date',       ''),
 				array('enable_bbcode',       1,                      ''),
 
 				array('enable_smilies',      'posts.use_emo',         ''),
 				array('enable_sig',          'posts.use_sig',         ''),
 				array('enable_magic_url',    1,                       ''),
 				array('post_username',       'posts.author_name',     'phpbb_set_encoding'),
 				array('post_subject',        'posts.post_title',      'phpbb_set_encoding'),
 				array('post_attachment',     'posts.attach_id',       'phpbb_has_attachement'),
 				array('post_edit_time',      'posts.edit_time',       array('typecast' => 'int')),
 				array('post_edit_count',     '',                      ''),
 				array('post_edit_reason',    '',                      ''),
 				array('post_edit_user',      '',                      'phpbb_post_edit_user'),
 			
 				array('bbcode_uid',          'posts.post_date',       'make_uid'),
 				array('post_text',           'posts.post',            'phpbb_prepare_message'),
 
 				array('bbcode_bitfield',     '',                      'get_bbcode_bitfield'),
 				array('post_checksum',       '',                      ''),
 			
 				// Commented out inline search indexing, this takes up a LOT of time. :D
 				// @todo We either need to enable this or call the rebuild search functionality post convert
 				// array('',                  '',                           'search_indexing'),
 				// array('',                  '',   ''),
 				// array('',                  '',   ''),
 
 			),
			
		)
	);
}

